﻿using System;
using System.Collections.Generic;

namespace VisuIT.ASLL.Example
{
	public class Creator
	{
		private const string DataPath = "/VisuIT/Data/";

		public Creator()
		{
			ASRoot.SetLicenseKey("EHB63-D3M49-E4028-2P7FK-L7T82");
		}

		private System.Globalization.CultureInfo NumberFormat => System.Globalization.CultureInfo.InvariantCulture;

		/// <summary>
		/// Writes the given autosar model into the given file.
		/// </summary>
		public void WriteAutosar(AUTOSAR autosar, string filename)
		{
			ASRoot.WriteToFile(autosar, filename, XmlHelper.Encoding.Utf8);
		}

		/// <summary>
		/// Creates an example autosar model. Not all required properties are set to form a valid
		/// autosar model. The autosar model serves only the purpose of demonstrating the usage of
		/// the parser model.
		/// </summary>
		public AUTOSAR CreateAutosar()
		{
			return new AUTOSAR()
			{
				FILE_INFO_COMMENT = CreateFileInfoComment(),
				AR_PACKAGES = new AUTOSARAR_PACKAGES()
				{
					AR_PACKAGE = new List<AR_PACKAGE>()
					{
						CreateRootPackage()
					}
				}
			};
		}

		private static FILE_INFO_COMMENT CreateFileInfoComment()
		{
			return new FILE_INFO_COMMENT()
			{
				SDGS = new FILE_INFO_COMMENTSDGS()
				{
					SDG = new List<SDG>()
					{
						new SDG()
						{
							GID = "META",
							// See property documentation for list of possible element types.
							Choice = new List<object>()
							{
								new SD()
								{
									GID = "PRODUCER",
									Value = "ASLL Example"
								},
								new SD()
								{
									GID = "PRODUCER_VERSION",
									Value = "1.33.7"
								}
							}
						}
					}
				}
			};
		}

		private AR_PACKAGE CreateRootPackage()
		{
			return new AR_PACKAGE()
			{
				SHORT_NAME = CreateIdentifier("VisuIT"),
				UUID = Guid.NewGuid().ToString(),
				AR_PACKAGES = new AR_PACKAGEAR_PACKAGES()
				{
					AR_PACKAGE = new List<AR_PACKAGE>()
					{
						CreateComponentPackage(),
						CreateDataPackage()
					}
	}
			};
		}

		private AR_PACKAGE CreateComponentPackage()
		{
			AR_PACKAGE package = CreateIdentifiable<AR_PACKAGE>("Components");
			package.ELEMENTS = new AR_PACKAGEELEMENTS()
			{
				// See property documentation for list of possible element types.
				Choice = new List<object>()
				{
					CreateApplicationComponent()
				}
			};

			return package;
		}

		private APPLICATION_SW_COMPONENT_TYPE CreateApplicationComponent()
		{
			APPLICATION_SW_COMPONENT_TYPE appComponent = CreateIdentifiable<APPLICATION_SW_COMPONENT_TYPE>("App");
			appComponent.PORTS = new APPLICATION_SW_COMPONENT_TYPEPORTS()
			{
				Choice = new List<ARIdentifiable>()
				{
					CreatePort("Measurement")
				}
			};

			return appComponent;
		}

		private PR_PORT_PROTOTYPE CreatePort(string name)
		{
			PR_PORT_PROTOTYPE port = CreateIdentifiable<PR_PORT_PROTOTYPE>(name);
			port.PROVIDED_REQUIRED_INTERFACE_TREF = new PR_PORT_PROTOTYPEPROVIDED_REQUIRED_INTERFACE_TREF
			{
				DEST = PORT_INTERFACE__SUBTYPES_ENUM.SENDER_RECEIVER_INTERFACE,
				Value = DataPath + name
			};

			return port;
		}

		private AR_PACKAGE CreateDataPackage()
		{
			AR_PACKAGE package = CreateIdentifiable<AR_PACKAGE>("Data");
			package.ELEMENTS = new AR_PACKAGEELEMENTS()
			{
				// See property documentation for list of possible element types.
				Choice = new List<object>()
				{
					CreateSenderReceiverInterface("Measurement"),
					CreateApplicationDataType("ApplDataType")
				}
			};

			return package;
		}

		private SENDER_RECEIVER_INTERFACE CreateSenderReceiverInterface(string name)
		{
			SENDER_RECEIVER_INTERFACE srInterface = CreateIdentifiable<SENDER_RECEIVER_INTERFACE>(name);
			srInterface.DESC = new MULTI_LANGUAGE_OVERVIEW_PARAGRAPH()
			{
				L_2 = new List<L_OVERVIEW_PARAGRAPH>()
				{
					new L_OVERVIEW_PARAGRAPH()
					{
						// See property documentation for list of possible element types.
						Choice = new List<object>()
						{
							// XString is a helper class to set text into mixed content.
							new XmlHelper.XString("Some text describing the measurement.")
						}
					}
				}
			};
			VARIABLE_DATA_PROTOTYPE dataPrototype = CreateIdentifiable<VARIABLE_DATA_PROTOTYPE>(name);
			dataPrototype.TYPE_TREF = new VARIABLE_DATA_PROTOTYPETYPE_TREF()
			{
				DEST = AUTOSAR_DATA_TYPE__SUBTYPES_ENUM.APPLICATION_PRIMITIVE_DATA_TYPE,
				Value = DataPath + "ApplDataType"
			};
			srInterface.DATA_ELEMENTS = new SENDER_RECEIVER_INTERFACEDATA_ELEMENTS()
			{
				VARIABLE_DATA_PROTOTYPE = new List<VARIABLE_DATA_PROTOTYPE>()
				{
					dataPrototype
				}
			};

			return srInterface;
		}

		private APPLICATION_PRIMITIVE_DATA_TYPE CreateApplicationDataType(string name)
		{
			APPLICATION_PRIMITIVE_DATA_TYPE dataType = CreateIdentifiable<APPLICATION_PRIMITIVE_DATA_TYPE>(name);
			dataType.SW_DATA_DEF_PROPS = new SW_DATA_DEF_PROPS()
			{
				SW_DATA_DEF_PROPS_VARIANTS = new SW_DATA_DEF_PROPSSW_DATA_DEF_PROPS_VARIANTS()
				{
					SW_DATA_DEF_PROPS_CONDITIONAL = new List<SW_DATA_DEF_PROPS_CONDITIONAL>()
					{
						new SW_DATA_DEF_PROPS_CONDITIONAL()
						{
							DISPLAY_FORMAT = new DISPLAY_FORMAT_STRING()
							{
								Value = "%6.3f"
							}
						}
					}
				}
			};

			return dataType;
		}

		private TIdentifiable CreateIdentifiable<TIdentifiable>(string name)
			where TIdentifiable : ARIdentifiable, new()
		{
			return new TIdentifiable()
			{
				SHORT_NAME = CreateIdentifier(name),
				UUID = Guid.NewGuid().ToString()
			};
		}

		private IDENTIFIER CreateIdentifier(string value)
		{
			return new IDENTIFIER()
			{
				Value = value,
				S = value.GetHashCode().ToString(this.NumberFormat)
			};
		}
	}
}
